from flask import Blueprint, request, render_template, redirect
from definitions.models import Asset, db
from config import item_attributes
from sqlalchemy import exc  # Import exc for database exceptions
from definitions.attribute import selectAttribute, intAttribute  # Import attribute classes

update_bp = Blueprint('editasset', __name__)

@update_bp.route('/update/<string:primary_value>/', methods=['GET', 'POST'])
def update(primary_value):
    # Identify the primary attribute
    primary_attrib = next(
        (attrib for attrib in item_attributes if attrib.primary),
        None
    )

    if not primary_attrib:
        return "Primary attribute not defined in configuration."

    # Fetch the item using the primary attribute
    item = Asset.query.filter_by(**{primary_attrib.attrib_name: primary_value}).first()
    if not item:
        return f"{primary_attrib.display_name} '{primary_value}' not found."

    if request.method == 'GET':
        return render_template(
            'update.html',
            item=item,
            item_attributes=item_attributes,
            isinstance=isinstance,  # Pass isinstance to the template
            hasattr=hasattr,  # Pass hasattr to the template
            selectAttribute=selectAttribute,  # Pass selectAttribute to the template
            intAttribute=intAttribute  # Pass intAttribute to the template
        )

    if request.method == 'POST':
        # Dynamically get form data using item_attributes
        form_data = {attrib.attrib_name: request.form[attrib.attrib_name] for attrib in item_attributes}

        # Validate status (if it's an option field)
        status_attrib = next((attrib for attrib in item_attributes if attrib.attrib_name == 'status'), None)
        if status_attrib and isinstance(status_attrib, selectAttribute):
            if form_data['status'] not in status_attrib.options:
                return render_template('update.html', item=item, exc='status', item_attributes=item_attributes)

        # Convert staffnum to int (if it's a number field)
        staffnum_attrib = next((attrib for attrib in item_attributes if attrib.attrib_name == 'staffnum'), None)
        if staffnum_attrib and isinstance(staffnum_attrib, intAttribute):
            try:
                form_data['staffnum'] = int(form_data['staffnum'])
            except ValueError:
                return render_template('update.html', item=item, exc='staffnum', item_attributes=item_attributes)

        # Update the item with the new data
        for attrib in item_attributes:
            setattr(item, attrib.attrib_name, form_data[attrib.attrib_name])

        try:
            db.session.commit()
        except exc.IntegrityError:
            return render_template('update.html', item=item, exc='integrity', item_attributes=item_attributes)
        except (exc.StatementError, exc.InvalidRequestError):
            return render_template('update.html', item=item, exc='status', item_attributes=item_attributes)

        return redirect('/viewall')